package cn.fw.valhalla.service.bus.follow;

import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.common.utils.StringUtils;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.customer.CustomerBaseInfo;
import cn.fw.valhalla.domain.db.follow.FollowNoticeRecord;
import cn.fw.valhalla.domain.db.follow.FollowTask;
import cn.fw.valhalla.domain.db.pool.CustomerCluePool;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.domain.enums.SendStatusEnum;
import cn.fw.valhalla.domain.enums.SettingTypeEnum;
import cn.fw.valhalla.domain.enums.SettingUnitEnum;
import cn.fw.valhalla.domain.vo.setting.SettingVO;
import cn.fw.valhalla.rpc.angel.InsurerRpcService;
import cn.fw.valhalla.rpc.angel.dto.InsuranceDTO;
import cn.fw.valhalla.rpc.ehr.EhrRpcService;
import cn.fw.valhalla.rpc.ehr.dto.StaffInfoDTO;
import cn.fw.valhalla.rpc.oop.OopService;
import cn.fw.valhalla.rpc.oop.dto.ShopDTO;
import cn.fw.valhalla.rpc.passport.TemplateMessageService;
import cn.fw.valhalla.rpc.passport.dto.RenewInsurNoticeMessageParam;
import cn.fw.valhalla.rpc.sms.SmsRpcService;
import cn.fw.valhalla.service.bus.setting.SettingBizService;
import cn.fw.valhalla.service.data.*;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.*;

import static cn.fw.valhalla.service.bus.follow.strategy.AbstractFollowStrategy.getCalendarType;

/**
 * @author : kurisu
 * @className : SendNoticeService
 * @description : 发送消息
 * @date: 2020-08-26 15:48
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class NoticeBizService {
    private final TemplateMessageService templateMessageService;
    private final FollowNoticeRecordService followNoticeRecordService;
    private final SettingBizService settingBizService;
    private final InsurerRpcService insurerRpcService;
    private final CustomerService customerService;
    private final CustomerBaseInfoService customerBaseInfoService;
    private final OopService oopService;
    private final CustomerCluePoolService customerCluePoolService;
    private final SmsRpcService smsRpcService;
    private final FollowTaskService followTaskService;
    private final EhrRpcService ehrRpcService;

    private final String type = "d";
    private final int day = 2;

    @Value("${follow.FmTemplateCode}")
    @Getter
    private String fmTemplateCode;

    @Value("${follow.RmTemplateCode}")
    @Getter
    private String rmTemplateCode;

    @Value("${follow.IrTemplateCode}")
    @Getter
    private String irTemplateCode;


    /**
     * 发送模板消息
     */
    public void sendNotice() {
        List<FollowNoticeRecord> list = followNoticeRecordService.list(Wrappers.<FollowNoticeRecord>lambdaQuery()
                .eq(FollowNoticeRecord::getStatus, SendStatusEnum.NOT_SENT)
                .le(FollowNoticeRecord::getSendTime, DateUtil.localDateTime2Date(LocalDateTime.now()))
                .last("limit 0, 100")
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        for (FollowNoticeRecord record : list) {
            sendNoticeAndCreate(record);
        }
    }

    /**
     * 推送模板消息 完成后生成下一次推送
     *
     * @param record
     */
    private void sendNoticeAndCreate(FollowNoticeRecord record) {
        CustomerCluePool cluePool = customerCluePoolService.getById(record.getClueId());
        if (Objects.isNull(cluePool)) {
            return;
        }
        Customer customer = customerService.queryById(cluePool.getRefererId());
        if (Objects.isNull(customer)) {
            createNextNotice(record, cluePool.getGroupId());
            return;
        }
        CustomerBaseInfo baseInfo = customerBaseInfoService.queryById(customer.getBaseId());
        if (Objects.isNull(baseInfo)) {
            createNextNotice(record, cluePool.getGroupId());
            return;
        }
        ShopDTO shop = oopService.shop(customer.getShopId());
        String shopName = Objects.isNull(shop) ? "" : shop.getShortName();
        boolean flag = false;
        if (FollowTypeEnum.IR.equals(record.getFollowType())) {
            flag = sendIR(record, cluePool, baseInfo, shopName);
        }
        if (FollowTypeEnum.FM.equals(record.getFollowType())) {
            flag = sendFM(record, cluePool, baseInfo, shopName);
        }
        if (FollowTypeEnum.RM.equals(record.getFollowType())) {
            flag = sendRM(record, cluePool, baseInfo, shopName);
        }
        if (flag) {
            createNextNotice(record, cluePool.getGroupId());
        }
    }

    private boolean sendIR(final FollowNoticeRecord record, final CustomerCluePool cluePool, CustomerBaseInfo baseInfo, String shopName) {
        if (DateUtil.sub(new Date(), record.getSendTime(), type) > day) {
            record.setStatus(SendStatusEnum.SUCCESS);
            return followNoticeRecordService.updateById(record);
        }
        Customer customer = Optional.ofNullable(customerService.getById(cluePool.getRefererId())).orElse(new Customer());
        RenewInsurNoticeMessageParam param = new RenewInsurNoticeMessageParam();
        param.setMemberId(baseInfo.getMemberId());
        param.setShopName(shopName);
        param.setExpireDate(DateUtil.getFullDateString(customer.getInsuranceExpires()));
        InsuranceDTO dto = insurerRpcService.queryInsByCustId(cluePool.getRefererId());
        if (Objects.nonNull(dto)) {
            param.setInsurCompanyName(dto.getInsurerName());
        }
        String result = templateMessageService.sendRINotice(param);
        record.setStatus(StringUtils.isEmpty(result) ? SendStatusEnum.SUCCESS : SendStatusEnum.FAILED);
        record.setStatusDesc(result);
        if (SendStatusEnum.FAILED.equals(record.getStatus())) {
            Long staffId = cluePool.getOriginalUserId();
            FollowTask task = followTaskService.queryOngoingTaskByClueId(cluePool.getId());
            if (Objects.nonNull(task)) {
                staffId = task.getFollowUser();
            }
            StaffInfoDTO infoDTO = ehrRpcService.queryStaffInfo(staffId);
            if (Objects.nonNull(infoDTO)) {
                Map<String, Object> map = new HashMap<>(4);
                map.put("date", DateUtil.getStringDateShort(customer.getInsuranceExpires()));
                map.put("shopName", shopName);
                map.put("userName", infoDTO.getName());
                map.put("phone", infoDTO.getMobile());
                smsRpcService.sendSms(getIrTemplateCode(), baseInfo.getMobile(), map, cluePool.getOriginalShopId());
            }
        }
        return followNoticeRecordService.updateById(record);
    }

    private boolean sendFM(final FollowNoticeRecord record, final CustomerCluePool cluePool, CustomerBaseInfo baseInfo, String shopName) {
        if (DateUtil.sub(new Date(), record.getSendTime(), type) > day) {
            record.setStatus(SendStatusEnum.SUCCESS);
            return followNoticeRecordService.updateById(record);
        }
        RenewInsurNoticeMessageParam param = new RenewInsurNoticeMessageParam();
        param.setMemberId(baseInfo.getMemberId());
        param.setShopName(shopName);
        param.setExpireDate(DateUtil.getFullDateString(cluePool.getDeadline()));

        String result = templateMessageService.sendFMNotice(param);
        record.setStatus(StringUtils.isEmpty(result) ? SendStatusEnum.SUCCESS : SendStatusEnum.FAILED);
        record.setStatusDesc(result);
        if (SendStatusEnum.FAILED.equals(record.getStatus())) {
            Long staffId = cluePool.getOriginalUserId();
            FollowTask task = followTaskService.queryOngoingTaskByClueId(cluePool.getId());
            if (Objects.nonNull(task)) {
                staffId = task.getFollowUser();
            }
            StaffInfoDTO infoDTO = ehrRpcService.queryStaffInfo(staffId);
            if (Objects.nonNull(infoDTO)) {
                Map<String, Object> map = new HashMap<>(3);
                map.put("shopName", shopName);
                map.put("userName", infoDTO.getName());
                map.put("phone", infoDTO.getMobile());
                smsRpcService.sendSms(getIrTemplateCode(), baseInfo.getMobile(), map, cluePool.getOriginalShopId());
            }
        }
        return followNoticeRecordService.updateById(record);
    }

    private boolean sendRM(final FollowNoticeRecord record, final CustomerCluePool cluePool, CustomerBaseInfo baseInfo, String shopName) {
        if (DateUtil.sub(new Date(), record.getSendTime(), type) > day) {
            record.setStatus(SendStatusEnum.SUCCESS);
            return followNoticeRecordService.updateById(record);
        }
        RenewInsurNoticeMessageParam param = new RenewInsurNoticeMessageParam();
        param.setMemberId(baseInfo.getMemberId());
        param.setShopName(shopName);
        param.setExpireDate(DateUtil.getFullDateString(cluePool.getDeadline()));

        String result = templateMessageService.sendRMNotice(param);
        record.setStatus(StringUtils.isEmpty(result) ? SendStatusEnum.SUCCESS : SendStatusEnum.FAILED);
        record.setStatusDesc(result);
        if (SendStatusEnum.FAILED.equals(record.getStatus())) {
            Long staffId = cluePool.getOriginalUserId();
            FollowTask task = followTaskService.queryOngoingTaskByClueId(cluePool.getId());
            if (Objects.nonNull(task)) {
                staffId = task.getFollowUser();
            }
            StaffInfoDTO infoDTO = ehrRpcService.queryStaffInfo(staffId);
            if (Objects.nonNull(infoDTO)) {
                Integer m = DateUtil.sub(cluePool.getAddTime(), new Date(), "m");
                Map<String, Object> map = new HashMap<>(4);
                map.put("num", Math.abs(m));
                map.put("shopName", shopName);
                map.put("userName", infoDTO.getName());
                map.put("phone", infoDTO.getMobile());
                smsRpcService.sendSms(getIrTemplateCode(), baseInfo.getMobile(), map, cluePool.getOriginalShopId());
            }
        }
        return followNoticeRecordService.updateById(record);
    }

    private void createNextNotice(final FollowNoticeRecord record, Long groupId) {
        Optional<SettingVO> timesSetting = settingBizService.querySettingByType(record.getFollowType(), SettingTypeEnum.NOTICE_TIMES, groupId);
        if (!timesSetting.isPresent()) {
            return;
        }
        int count = followNoticeRecordService.count(Wrappers.<FollowNoticeRecord>lambdaQuery().eq(FollowNoticeRecord::getClueId, record.getClueId()));
        int value = Optional.ofNullable(timesSetting.get().getDetailValue()).orElse(0);
        if (count >= value) {
            return;
        }
        Optional<SettingVO> cycleSetting = settingBizService.querySettingByType(record.getFollowType(), SettingTypeEnum.NOTICE_CYCLE, groupId);
        cycleSetting.ifPresent(r -> {
            Timestamp expired = DateUtil.getExpired(record.getSendTime(), r.getDetailValue(), getCalendarType(Objects.requireNonNull(SettingUnitEnum.ofValue(r.getUnit()))));
            followNoticeRecordService.save(createEntity(record, expired));
        });
    }

    private FollowNoticeRecord createEntity(final FollowNoticeRecord record, Timestamp time) {
        FollowNoticeRecord noticeRecord = new FollowNoticeRecord();
        noticeRecord.setCustomerId(record.getCustomerId());
        noticeRecord.setClueId(record.getClueId());
        noticeRecord.setFollowType(record.getFollowType());
        noticeRecord.setStatus(SendStatusEnum.NOT_SENT);
        noticeRecord.setSendTime(time);
        noticeRecord.setCreateTime(DateUtil.localDateTime2Date(LocalDateTime.now()));
        return noticeRecord;
    }
}
